Package: Rglpk
Version: 0.6-5.1
Title: R/GNU Linear Programming Kit Interface
Description: R interface to the GNU Linear Programming Kit.
  'GLPK' is open source software for solving large-scale linear programming (LP),
  mixed integer linear programming ('MILP') and other related problems.
Authors@R: c(person("Stefan", "Theussl", role = c("aut", "cre"), email = "Stefan.Theussl@R-project.org", comment = c(ORCID = "0000-0002-6523-4620")),
             person("Kurt", "Hornik", role = "aut", comment = c(ORCID = "0000-0003-4198-9911")),
	     person("Christian", "Buchta", role = "ctb"),
	     person("Florian", "Schwendinger", role = "ctb", comment = c(ORCID = "0000-0002-3983-9773")),
	     person("Heinrich", "Schuchardt", role = "ctb"))
Depends: slam (>= 0.1-9)
SystemRequirements: GLPK library package (e.g., libglpk-dev on
        Debian/Ubuntu)
License: GPL-2 | GPL-3
URL: https://R-Forge.R-project.org/projects/rglp/,
        https://www.gnu.org/software/glpk/
NeedsCompilation: yes
Packaged: 2024-01-13 15:39:28 UTC; hornik
Author: Stefan Theussl [aut, cre] (<https://orcid.org/0000-0002-6523-4620>),
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Christian Buchta [ctb],
  Florian Schwendinger [ctb] (<https://orcid.org/0000-0002-3983-9773>),
  Heinrich Schuchardt [ctb]
Maintainer: Stefan Theussl <Stefan.Theussl@R-project.org>
Repository: CRAN
Date/Publication: 2024-01-13 16:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:03:31 UTC; unix
Archs: Rglpk.so.dSYM
